/*********************************************************

Author: Hubert Gschwandtner <hubert@schorsch.at> www.schorsch.at
Basic routines for interfacing a LCD Type VLGEM1021-03 
Compatible with HB24208 / HB24209 
LCD-Controller KS0074 or compatible
This library is based on the HD44780 LCD-Lib of P.Fleury
Adapted to the SPI-Interface and the KS0074 Controller
This routine uses SPI-Mode 0
Version 1.0.0 


*********************************************************/



#include <avr/io.h>
#include "spi_lcd.h"



	

#define delay(us)  _delayFourCycles( ( ( 1*(F_CPU/4000) )*us)/1000 )

/*************************************************************************
 delay loop for small accurate delays: 16-bit counter, 4 cycles/loop
*************************************************************************/
static inline void _delayFourCycles(unsigned int __count)
{
    if ( __count == 0 )    
        __asm__ __volatile__( "rjmp 1f\n 1:" );    // 2 cycles
    else
        __asm__ __volatile__ (
    	    "1: sbiw %0,1" "\n\t"                  
    	    "brne 1b"                              // 4 cycles/loop
    	    : "=w" (__count)
    	    : "0" (__count)
    	   );
}



void lcd_init(unsigned char dispAttr) {
	SPI_PORT = (1<<MOSI)|(1<<SCK)|(1<<SS);
	SPCR = (1<<SPE) | (1<<MSTR) |(1<<DORD) ;
	SPCR |= (SPR0x<<SPR0)|(SPR1x<<SPR1);	/* SCK-Frequenz */
	SPSR = (SPI2Xx<<SPI2X);
	delay(15000);
	delay(15000);
	slcd_command(0x1f);	/* Start Init Command */
	delay(50);
	lcd_data(dispAttr);	/* Display on, Cursor on, Blink off */
	delay(50);
	lcd_data(1<<LCD_CLR);	/* Clear Display */
	delay(2000);
	slcd_command(0x5f);	/* Umschalten auf Send Data */
}


void slcd_command(unsigned char scmd){ /* Send intern Disp-command */
	SPDR = scmd;
	while(!(SPSR & (1<<SPIF)));
	delay(50);
}

void lcd_command(const unsigned char cmd){
	slcd_command(0x1f);
	lcd_data(cmd);
	slcd_command(0x5f);
}

/* Send Disp-Data */
void lcd_data(unsigned char data){		
	unsigned char temp = data;
	data &= 0x0f;
	SPDR = data;
	while(!(SPSR & (1<<SPIF)));/* Wait for transmission complete */
	temp = temp >>4;
	temp &= 0x0f;
	SPDR = temp;
	while(!(SPSR & (1<<SPIF)));/* Wait for transmission complete */
	delay(50);
}

void lcd_putc(unsigned char c){	/* Print character on LCD (no auto linefeed)*/
	lcd_data(c);
}

/* Clear Display and set cursor to home position*/
void lcd_clrscr(void){ 		
	slcd_command(0x1f);
	lcd_data(0x01);	
	delay(2000);
	slcd_command(0x5f);

}

/* print string on lcd (no auto linefeed) */
void lcd_puts(const char *s)	
{
    register  char c;

    while ( (c = *s++) ) {
        lcd_putc(c);
}	
}

/* Set cursor to home position */
void lcd_home(void){
	slcd_command(0x1f);
	lcd_data(0x02);	
	delay(2000);
	slcd_command(0x5f);
}

/*Set cursor to specified position */
void lcd_gotoxy(unsigned char x, unsigned char y){

#if LCD_LINES==1
    lcd_command((1<<LCD_DDRAM)+LCD_START_LINE1+x);
#endif
#if LCD_LINES==2
    if ( y==0 ) 
        lcd_command((1<<LCD_DDRAM)+LCD_START_LINE1+x);
    else
        lcd_command((1<<LCD_DDRAM)+LCD_START_LINE2+x);
#endif
#if LCD_LINES==4
    if ( y==0 )
        lcd_command((1<<LCD_DDRAM)+LCD_START_LINE1+x);
    else if ( y==1)
        lcd_command((1<<LCD_DDRAM)+LCD_START_LINE2+x);
    else /* y==3 */
        lcd_command((1<<LCD_DDRAM)+LCD_START_LINE4+x);
#endif
}

#if print_string_from_flash 
/* print string from program memory on lcd (no auto linefeed) */
void lcd_puts_p(const char *progmem_s){
    register char c;
    while ( (c = pgm_read_byte(progmem_s++)) ) {
        lcd_putc(c);
    }
}
#endif

#if print_string_from_EEPROM
/* print string from EEPROM on lcd (no auto linefeed) */
void lcd_puts_eep(const unsigned char *eep){
    register unsigned char c;
    while ( (c = eeprom_read_byte(eep++)) ) {
        lcd_putc(c);
    }
}
#endif

#if Store_individual_character_from_flash_on_cgram
/* Store individual character from flash on cgram */
void lcd_putc_cgram_p(const char *progmem_s, unsigned char _position){
	lcd_command((1<<LCD_CGRAM)+_position*8);
	register char c;
	register char x1;
    for (x1=0;x1<=7;x1++){ 
		(c = pgm_read_byte(progmem_s++));  
        lcd_putc(c);
		}
	lcd_clrscr();
}
#endif

#if Store_individual_character_from_EEPROM_on_cgram
/* Store individual character from EEPROM on cgram */
void lcd_putc_cgram_eep(const unsigned char *eep, unsigned char _position){
	lcd_command((1<<LCD_CGRAM)+_position*8);
	register unsigned char c;
	register char x1;
    for (x1=0;x1<=7;x1++){
		(c = eeprom_read_byte(eep++));  
        lcd_putc(c);
	}
	lcd_clrscr();
}
#endif


